<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://post.lt
 * @since      1.0.0
 *
 * @package    Woo_Lithuaniapost
 * @subpackage Woo_Lithuaniapost/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Woo_Lithuaniapost
 * @subpackage Woo_Lithuaniapost/public
 * @author     AB "Lietuvos Paštas" <info@post.lt>
 */
class Woo_Lithuaniapost_Public
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct ( $plugin_name, $version )
    {
		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles ()
    {
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Woo_Lithuaniapost_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Woo_Lithuaniapost_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style ( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/woo-lithuaniapost-public.css', array(), $this->version, 'all' );
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
    {
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Woo_Lithuaniapost_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Woo_Lithuaniapost_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script ( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/woo-lithuaniapost.js', array( 'jquery' ), $this->version, false );
        wp_localize_script( $this->plugin_name, 'woo_lithuaniapost', array( 'ajax_url' => admin_url ( 'admin-ajax.php' ) ) );
    }

	/**
	 * Get formatted LPEXPRESS terminal list
	 *
	 * @return array
	 * @since 1.0.0
	 */
	protected function get_terminal_list ()
    {
        $shipping_country_code = $_POST['s_country'] ?? WC()->customer->get_shipping_country();
		return apply_filters("woo_lithuaniapost_terminal_service_get_terminals_by_country_code", $shipping_country_code);
    }

	protected function get_selected_terminal_city ()
    {
        $selected_terminal_session = WC()->session->get('selected_lpexpress_terminal');
        return !empty ( $selected_terminal_session ) ? $selected_terminal_session [ 'city' ] : null;
    }

	protected function get_selected_terminal_id ()
    {
        $selected_terminal_session = WC()->session->get('selected_lpexpress_terminal');
        return !empty ( $selected_terminal_session ) ? $selected_terminal_session [ 'terminal_id' ] : null;
    }

	protected function get_selected_terminal ()
    {
        $selected_terminal_session = WC()->session->get('selected_lpexpress_terminal');
        return !empty ( $selected_terminal_session ) ? $selected_terminal_session [ 'terminal' ] : null;
    }

    /**
     * Render field for UNISEND terminal
     *
     * @param $method
     * @param int $index
     * @since 1.0.0
     */
    public function render_terminal_field ( $method, $index )
    {
        if (!$this->is_selected_method_lp($method, $index)) {
            return;
        }

        // Check if selected method is UNISEND terminal
        if ($this->get_method_settings_value($method->get_instance_id(), 'plan') == 'TERMINAL') {
            include __DIR__ . '/partials/html-lpexpress-terminal.php';
        }
    }

    /**
     * Render delivery time
     *
     * @param $method
     * @param $index
     * @since 1.0.0
     */
    public function render_delivery_time($method, $index)
    {
        if (!is_checkout() || $method->get_method_id() != 'woo_lithuaniapost_lpexpress_terminal') {
            return;
        }
        $delivery_time = $this->get_method_settings_value($method->get_instance_id(), 'delivery_time');
        if ($delivery_time) {
            include __DIR__ . '/partials/html-delivery-time.php';
        }
    }

    public function validate_shipping($arg)
    {
        if (!$this->is_chosen_method_lp() || !is_checkout()) return;
        $shipping_plan = $this->get_chosen_method_settings_value('plan');
        if ($shipping_plan == 'TERMINAL') {
            if (!$this->validate_selected_terminal($arg)) return;
        }
        $this->validate_parcel($arg);
    }

    private function create_delivery_address_validation_request(string $delivery_address_type = 'shipping'): array
    {
        $terminal_id = $this->get_selected_terminal_id() ?? null;
        $country_code = $_POST[$delivery_address_type . '_country'];
        $address = $country_code == 'LT' ? $_POST[$delivery_address_type . '_address_1'] . ' ' . $_POST[$delivery_address_type . '_address_2'] : null;
        return [
            'name' => sprintf('%s %s',
                $_POST[$delivery_address_type . '_first_name'],
                $_POST[$delivery_address_type . '_last_name']
            ),
            'companyName' => $_POST[$delivery_address_type . '_company'],
            'contacts' => [
                'phone' => @$_POST['billing_phone'],
                'email' => @$_POST['billing_email']
            ],
            'address' => [
                'terminalId' => $terminal_id,
                'locality' => $_POST[$delivery_address_type . '_city'],
                'address' => $address,
                'address1' => $_POST[$delivery_address_type . '_address_1'],
                'address2' => $_POST[$delivery_address_type . '_address_2'],
                'postalCode' => $_POST[$delivery_address_type . '_postcode'],
                'countryCode' => $country_code
            ],
        ];
    }

    private function handle_validation_error_response($response_body): void
    {
        if ($response_body && $response_body->errors && is_array($response_body->errors)) {
            foreach ($response_body->errors as $error_response) {
                $this->display_validation_error($error_response);
            }
        } else if (isset($response_body->error)) {
            $this->display_validation_error($response_body->error);
        } else if (is_array($response_body)) {
            foreach ($response_body as $response_item) {
                if (isset($response_item->error)) {
                    $this->display_validation_error($response_item);
                }
            }
        }
    }

    private function display_validation_error($error_response): void
    {
        $error = @$error_response->error;
        $error_description = @$error_response->error_description;
        if ($error) {
            $error_field = $error_response->field ?? null;
            $error_message = $error_description ?? $error;
            if (strlen($error_field) > 0) {
                $error_field_parts = explode(".", $error_field);
                $last_error_field_part = end($error_field_parts);
                if ($last_error_field_part) {
                    $error_message = $last_error_field_part . ' ' . $error_message;
                }
            }
            wc_add_notice(__('Please fill correct shipping data: ' . $error_message, 'woo-lithuaniapost'), 'error');
        }
    }

	/**
	 * Save shipping method settings and/or selected terminal
	 *
	 * @param $order_id
	 * @since 1.0.0
	 */
	public function save_order_meta_info ( $order_id )
	{
        if ($this->is_chosen_method_lp()) {
            $order = wc_get_order($order_id);
            if (!$order) return;
            $shipping_method_instance_id = $this->get_chosen_method_instance_id();
            if ($shipping_method_instance_id) {
                $order->update_meta_data('_woo_lithuaniapost_lpexpress_shipping_method_instance_id', $shipping_method_instance_id);
            }

            $saved_terminal_id = $this->get_selected_terminal_id();
            // Save terminal id
            if (isset ($saved_terminal_id)) {

                $terminal_id = sanitize_text_field($saved_terminal_id);

                $terminal = apply_filters('woo_lithuaniapost_terminal_service_get_terminal_by_id', $terminal_id);

                if ($terminal) {
                    $order->update_meta_data('_woo_lithuaniapost_lpexpress_terminal_id', $terminal_id);
                    $order->update_meta_data('_woo_lithuaniapost_lpexpress_terminal', sprintf('%s - %s, %s', $terminal [0]->name, $terminal [0]->address, $terminal [0]->city));
                }
            }
            $order->save();
        }
    }

    /**
     * Save selected terminal to session
     * @since 1.0.0
     */
    public function save_selected_terminal_session ()
    {
        WC ()->session->set ( 'selected_lpexpress_terminal', [
                'city' => $_REQUEST [ 'city' ] ,
                'terminal' => $_REQUEST [ 'terminal' ],
                'terminal_id' => $_REQUEST [ 'terminal_id' ]
            ]
        );

        wp_die ();
    }

    /**
     * Display selected terminal to order email
     *
     * @param array $fields
     * @param bool $sent_to_admin
     * @param mixed order
     * @return mixed
     * @since 1.0.0
     */
    public function add_terminal_field_order_email ( $fields, $sent_to_admin, $order )
    {
        if ( $terminal_id = $order->get_meta('_woo_lithuaniapost_lpexpress_terminal_id') ) {

            $terminal = apply_filters('woo_lithuaniapost_terminal_service_get_terminal_by_id', $terminal_id);

            $fields [ 'selected_lpexpress_terminal' ] = [
                'label' => __( 'UNISEND Parcel Locker', 'woo-lithuaniapost' ),
                'value' => sprintf ( '%s - %s, %s', $terminal [ 0 ]->name, $terminal [ 0 ]->address, $terminal [ 0 ]->city )
            ];
        }

        return $fields;
    }

    /**
     * Display selected terminal to order email
     *
     * @param array $fields
     * @param bool $sent_to_admin
     * @param mixed order
     * @return mixed
     * @since 1.0.0
     */
    public function add_terminal_field_order_thankyou ( $order )
    {
        $order = wc_get_order ( $order );
        if ( $terminal_id = $order->get_meta('_woo_lithuaniapost_lpexpress_terminal_id') ) {

            $terminal = apply_filters('woo_lithuaniapost_terminal_service_get_terminal_by_id', $terminal_id);

            echo sprintf ( '<b>%s:</b> %s - %s, %s<br><br><br>', __( 'UNISEND Parcel Locker', 'woo-lithuaniapost' ), $terminal [ 0 ]->name, $terminal [ 0 ]->address, $terminal [ 0 ]->city );
        }
    }

    private function get_method_settings_value($instance_id, $key)
    {
        return get_option('woocommerce_woo_lithuaniapost_lpexpress_terminal_' . $instance_id . '_settings')[$key];
    }

    private function is_selected_method_lp($method, $index): bool
    {
        // Get selected method
        $selected_method = WC()->session->get('chosen_shipping_methods') [$index];
        if (!$selected_method) return false;
        return $method->get_id() == $selected_method && $method->get_method_id() == 'woo_lithuaniapost_lpexpress_terminal';
    }

    private function is_chosen_method_lp(): bool
    {
        $selected_method = WC()->session->get('chosen_shipping_methods');
        if (!$selected_method) return false;
        return strpos($selected_method[0], "woo_lithuaniapost_lpexpress_terminal") !== false;
    }

    private function get_chosen_method_settings_value($key)
    {
        $selected_method = WC()->session->get('chosen_shipping_methods');
        if (!$selected_method) return null;
        return $this->get_method_settings_value(explode(":", $selected_method[0])[1], $key);
    }

    private function get_chosen_method_instance_id()
    {
        $selected_method = WC()->session->get('chosen_shipping_methods');
        if (!$selected_method) return null;
        return explode(":", $selected_method[0])[1];
    }

    /**
     * Custom terminal field validation
     *
     * @since 1.0.0
     */
    private function validate_selected_terminal($arg): bool
    {
        $selected_terminal_id = $this->get_selected_terminal_id();
        if (empty ($selected_terminal_id)) {
            wc_add_notice(__('Please select parcel locker', 'woo-lithuaniapost'), 'error');
            return false;
        } else {
            // Validate against random values
            global $wpdb;

            $request_terminal_id = sanitize_text_field($selected_terminal_id);
            // Check if selected terminal exists in DB
            $terminal = apply_filters('woo_lithuaniapost_terminal_service_get_terminal_by_id', $request_terminal_id);
            if (!$terminal) {
                wc_add_notice(__('Please select parcel locker', 'woo-lithuaniapost'), 'error');
                return false;
            }
        }
        return true;
    }

    private function get_request_shipping_country(): string
    {
        $use_shipping_address = $_POST['ship_to_different_address'] ?? null == true;
        $prefix = $use_shipping_address ? 'shipping' : 'billing';
        return $_POST[$prefix . '_country'];
    }

    private function validate_parcel($arg): bool
    {
        $validation_option = Woo_Lithuaniapost_Admin_Settings::get_option('validate_shipping_before_checkout');
        if ($validation_option == 'no') {
            return true;
        }
        $validate_parcel_request = $this->create_parcel_validation_request();
        $error_callback = function ($response_body) {
            return $this->handle_validation_error_response($response_body);
        };
        return apply_filters('woo_lithuaniapost_api_is_parcel_valid', $validate_parcel_request, $error_callback);
    }

    private function create_parcel_validation_request()
    {

        $plan_code = $this->get_chosen_method_settings_value('plan');
        $parcel_type = $this->get_chosen_method_settings_value('parcel_type');
        $size = $this->get_chosen_method_settings_value('size') ?: null;
        $weight = 1;//weight should be already validated via available shipping check

        $services = null;
        $payment_method = $_POST['payment_method'];
        if ($payment_method == 'cod') {
            $services [] = ['code' => 'cod', 'value' => 1];
        }

        $use_shipping_address = $_POST['ship_to_different_address'] ?? null == true;
        $receiver_address = $this->create_delivery_address_validation_request($use_shipping_address ? 'shipping' : 'billing');

        return [
            'parcel' => [
                'size' => $size,
                'type' => $parcel_type,
                'weight' => $weight
            ],
            'plan' => [
                'code' => $plan_code
            ],
            'receiver' => $receiver_address,
            'services' => $services,
            'options' => [
                'resolveAddress' => true,
                'lookupAddress' => false,
                'validateSenderAddress' => false,
                'validateCn' => false
            ]
        ];
    }
}
